package sim;

import java.awt.*;
import java.awt.event.*;

import sim.lib.EditDelay;
import sim.util.DialogButton;
import sim.util.SimSeparator;
import sim.util.ImagePanel;

public class Settings extends Dialog implements ActionListener {

	private RandomProperties properties;
	private ImagePanel iconHolder;
	private static Image ICON;
	
	public static double delay = 0.00000001;

	public Settings() {
		super(MainWindow.MAIN_WINDOW, "Properties");

		if (Settings.ICON == null) {
			Settings.ICON = GuiFileLink.getImage("sim/images/Settings.png");
		}

		this.setLayout(new GridBagLayout());
		this.setModal(true);
		this.setResizable(false);

		Panel p = new Panel(new FlowLayout(FlowLayout.RIGHT, 10, 10));

		DialogButton b = new DialogButton("OK");
		b.addActionListener(this);
		p.add(b);
		b = new DialogButton("Cancel");
		b.addActionListener(this);
		p.add(b);

		GridBagConstraints c = new GridBagConstraints();
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.anchor = GridBagConstraints.EAST;
		c.fill = GridBagConstraints.NONE;

		this.add(p, c);

		this.properties = new RandomProperties();

		c.gridy = 1;
		c.insets = new Insets(10, 20, 40, 20);
		c.anchor = GridBagConstraints.CENTER;
		this.add(this.properties, c);

		p = new Panel(new FlowLayout(FlowLayout.LEFT, 10, 0));
		this.iconHolder = new ImagePanel(Settings.ICON);
		p.add(this.iconHolder);
		p.add(new Label("Settings", Label.LEFT));

		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0, 0, 0, 0);
		c.gridy = 0;

		this.add(p, c);

		this.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				setVisible(false);
			}
		});
	}

	public void setVisible(boolean visible) {
		if (visible) {
			this.properties.initialise();
		}

		super.setVisible(visible);
	}

	public void addNotify() {
		super.addNotify();

		int width = 40 + this.properties.getSize().width + this.getInsets().left + this.getInsets().right;
		int height = 70 + this.iconHolder.getPreferredSize().height + this.properties.getSize().height + DialogButton.HEIGHT + this.getInsets().top + this.getInsets().bottom;
		this.setSize(width, height);
		doLayout();

		this.setLocation(MainWindow.getMiddleOfScreen(this));
	}

	public void actionPerformed(ActionEvent e) {
		this.setVisible(false);

		if (((DialogButton) e.getSource()).getLabel().equals("OK")) {
			this.properties.ok();
		}
	}

	public void paint(Graphics g) {
		super.paint(g);

		Rectangle r = this.properties.getBounds();

		r.x = r.x - 10;
		r.y = r.y - 5;

		r.width = r.width + 20;
		r.height = r.height + 40;

		g.setColor(Color.white);
		g.drawLine(r.x, r.y, r.x + r.width - 2, r.y);
		g.drawLine(r.x, r.y, r.x, r.y + r.height - 2);

		g.setColor(Color.black);
		g.drawLine(r.x + r.width - 1, r.y, r.x + r.width - 1, r.y + r.height - 1);
		g.drawLine(r.x, r.y + r.height - 1, r.x + r.width - 1, r.y + r.height - 1);

		g.setColor(this.getBackground().darker());
		g.drawLine(r.x + r.width - 2, r.y + 1, r.x + r.width - 2, r.y + r.height - 2);
		g.drawLine(r.x + 1, r.y + r.height - 2, r.x + r.width - 2, r.y + r.height - 2);
	}

	private class RandomProperties extends Container implements ActionListener, FocusListener {

		private Checkbox editRandom = new Checkbox("Random delay");
		private TextField editDelay = new TextField();
		private double old;
		private Label random = new Label("Random");

		public RandomProperties() {
			super();
			this.setLayout(new BorderLayout(0, 15));
			
			// configuration
			Panel big = new Panel(new BorderLayout(0, 15));
			Panel p = new Panel(new BorderLayout());
			p.add(this.random, BorderLayout.WEST);
			p.add(new SimSeparator(), BorderLayout.CENTER);
			big.add(p, BorderLayout.NORTH);
			
			p = new Panel(new GridLayout(1, 2));
			p.add(this.editRandom);
			Panel p1 = new Panel(new GridLayout(1,2));
			p1.add(new Label("Less than"));
			p1.add(this.editDelay);
			p.add(p1);
			big.add(p, BorderLayout.CENTER);
			
			this.add(big, BorderLayout.NORTH);
		}

		public void addNotify() {
			super.addNotify();
			this.setSize(290, this.editRandom.getPreferredSize().height + this.random.getPreferredSize().height + 15);
		}

		public void initialise() {
			this.editRandom.setState(sim.Random.RANDOM);
			this.editDelay.setText(Double.toString(delay));
		}

		public void ok() {
			sim.Random.setState(this.editRandom.getState());
			SimMenuBar.RANDOM.setState(this.editRandom.getState());
			delay = Double.valueOf(this.editDelay.getText()).doubleValue();
		}

		public void actionPerformed(ActionEvent e) {
			double newDelay;
			
			try
			{
				newDelay = Double.valueOf(this.editDelay.getText()).doubleValue();
				
				if(newDelay < 0)
					this.editDelay.setText(Double.toString(this.old));
				else
					this.old = newDelay;
			}
			catch(NumberFormatException nfe)
			{
				this.editDelay.setText(Double.toString(this.old));
			}
		}

		public void focusGained(FocusEvent e) {
		}

		public void focusLost(FocusEvent e) {
			double newDelay;
			
			try
			{
				newDelay = Double.valueOf(this.editDelay.getText()).doubleValue();
				
				if(newDelay < 0)
					this.editDelay.setText(Double.toString(this.old));
				else
					this.old = newDelay;
			}
			catch(NumberFormatException nfe)
			{
				this.editDelay.setText(Double.toString(this.old));
			}
		}

		public Dimension getPreferredSize() {
			return this.getSize();
		}

		public Dimension getMinimumSize() {
			return this.getSize();
		}

		public Dimension getMaximumSize() {
			return this.getSize();
		}
	}
}
