package sim;

import java.awt.FlowLayout;
import java.awt.Panel;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;

import sim.util.SimButton;
import sim.util.SimButtonListener;

public class SettingsShortcut extends Panel {

	private SettingsShortcutListener listener;

	public SettingsShortcut() {
		super();
		this.setLayout(new FlowLayout(FlowLayout.LEFT, 8, 0));
	}

	public void addNotify() {
		super.addNotify();
		this.listener = new SettingsShortcutListener(this);
	}

	/* ==================================================================
	 Listener to handle mouse events
	 ================================================================= */
	private class SettingsShortcutListener extends SimButtonListener {
		private SimButton Settings;
		private boolean buttonUpOnLastMouseDown;
		private Settings settingsDialog = new Settings();

		public SettingsShortcutListener(SettingsShortcut parent) {
			Panel p = new Panel(new FlowLayout(FlowLayout.LEFT, 0, 0));
			parent.add(p);

			this.buttonUpOnLastMouseDown = true;
			this.Settings = new SimButton("sim/images/Settings.png", "Settings", NorthPanel.BUBBLE_WAIT);
			this.Settings.setListener(this);
			p.add(this.Settings);
		}

		public void actionPerformed(ActionEvent e) {
			this.settingsDialog.setVisible(true);
		}

		public void activate(SimButton ib) {
			ib.processActionEvent();
			ib.setArmed(false);
		}

		public void arm(SimButton ib) {
			ib.setArmed(true);
		}

		public void disarm(SimButton ib) {
			ib.setArmed(false);
		}

		public void mousePressed(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled()) {
				if (ib.isRaised()) {
					ib.paintInset();
					this.buttonUpOnLastMouseDown = false;
					this.arm(ib);
				} else {
					ib.paintRaised();
					this.buttonUpOnLastMouseDown = true;
					this.arm(ib);
				}
			}
		}

		public void mouseClicked(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled() && ib.isArmed()) {
				this.activate(ib);
			}
		}

		public void mouseReleased(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			Point pt = event.getPoint();

			if (ib.contains(pt.x, pt.y)) {
				if (ib.isEnabled()) {
					if (ib.isRaised()) {
						ib.paintInset();
						this.buttonUpOnLastMouseDown = false;
						this.arm(ib);
					} else {
						ib.paintRaised();
						this.buttonUpOnLastMouseDown = true;
						this.arm(ib);
					}
				}
				this.mouseClicked(event);
			}
		}

		public void mouseDragged(MouseEvent event) {
			SimButton ib = (SimButton) event.getSource();
			if (ib.isEnabled() && ib.isArmed()) {
				Point loc = event.getPoint();

				if (ib.contains(loc.x, loc.y)) {
					if (this.buttonUpOnLastMouseDown) {
						if (!ib.isRaised()) {
							ib.paintRaised();
						}
					} else if (ib.isRaised()) {
						ib.paintInset();
					}
				} else {
					if (this.buttonUpOnLastMouseDown) {
						if (ib.isRaised()) {
							ib.paintInset();
						}
					} else if (!ib.isRaised()) {
						ib.paintRaised();
					}
				}
			}
		}
	}
}