package sim.lib.functions;

import java.awt.*;

import sim.*;
import sim.engine.*;
import sim.lib.wires.Junction;

public abstract class Function2fat1thinIn1fat1thinOut extends WrapperPainted implements EngineModule
{
	protected double delay = 1;
	protected int busSize;
	protected boolean carryin = false;
	
	public double getDelay()
	{
		return this.delay;
	}
	
	public void changeDelay(double newValue)
	{
		this.delay = newValue;
	}
	
	public int getBusSize()
	{
		return this.busSize;
	}
	
	public void setBusSize(int size)
	{
		this.busSize = size;
	}
	
	public boolean getCarryIn()
	{
		return this.carryin;
	}
	
	public void setCarryIn(boolean carryIn)
	{
		this.carryin = carryIn;
	}

/* ==================================================================
	Creation Part
	================================================================= */
	
	public Wrapper createWrapper()
	{
		return this.getCopy();
	}
	
	public Wrapper createWrapper(Point gridPosition)
	{
		Wrapper result = this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}
	
	protected abstract Wrapper getCopy();
/* ==================================================================
	GUI part
	================================================================= */
	
	public Function2fat1thinIn1fat1thinOut()
	{
		super();
		this.busSize = 8;
	}
	
	public void initializeGridSize()
	{
		this.setGridSize(6, 4);
	}
	
/* ==================================================================
	Maintanance Part
	================================================================= */
	private Junction inputX = null;
	private Junction inputY = null;
	private Junction output = null;
	private Junction overflow = null;
	private Junction carryIn  = null;
	
	public boolean canDrop()
	{
		boolean result = Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 2, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y, this.busSize);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 3, this.gridLocation.y + 4, this.busSize);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 4, this.gridLocation.y, this.busSize);
		if(this.carryin)
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 6, this.gridLocation.y + 2, 1);
		return result;
	}
		
	public void droped()
	{
		this.overflow = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 2, 1);
		this.inputX = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y, this.busSize);
		this.inputY = Wrapper.setPinAt(this.gridLocation.x + 4, this.gridLocation.y, this.busSize);
		this.output = Wrapper.setPinAt(this.gridLocation.x + 3, this.gridLocation.y + 4, this.busSize);
		if(this.carryin)
			this.carryIn = Wrapper.setPinAt(this.gridLocation.x + 6, this.gridLocation.y + 2, 1);
		
		this.changeColor(Color.black);
		this.oldBusSize = 0;
	}
	
	public void selected()
	{
		this.output.removePin();
		this.overflow.removePin();
		this.inputX.removePin();
		this.inputY.removePin();
		if(this.carryin)
			this.carryIn.removePin();
		
		this.changeColor(Color.green);
	}
	
	public void checkAfterSelected()
	{
		Wrapper.checkPin(this.output);
		Wrapper.checkPin(this.overflow);
		Wrapper.checkPin(this.inputX);
		Wrapper.checkPin(this.inputY);
		if(this.carryin)
			Wrapper.checkPin(this.carryIn);
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	public void createEnginePeer(EnginePeerList epl)
	{
		int loop;		
		EnginePeer ep;
		if(this.carryin)
			ep= new EnginePeer(2 * this.busSize + 1, this.busSize + 1, this);
		else
			ep= new EnginePeer(2 * this.busSize, this.busSize + 1, this);
		
		for(loop = 0; loop < this.busSize; loop++)
		{
			ep.setInputPin(loop, this.inputX.getNodes().getItemAt(loop));
			ep.setInputPin(loop + this.busSize, this.inputY.getNodes().getItemAt(loop));
			ep.setOutputPin(loop, this.output.getNodes().getItemAt(loop));
		}
		
		ep.setOutputPin(this.busSize, this.overflow.getNodes().getItemAt(0));
		if(this.carryin)
			ep.setInputPin(2 * this.busSize, this.carryIn.getNodes().getItemAt(0));
		
		epl.insertItem(ep);
	}
	public void reset()
	{
	}
	
	public Wrapper getParentWrapper()
	{
		return this;
	}
	
/* ==================================================================
	Storage Part
	================================================================= */
	public String getSpecificParameters()
	{
		return (Double.toString(this.delay) + Wrapper.SEPARATOR + this.busSize + Wrapper.SEPARATOR + this.carryin + Wrapper.SEPARATOR);
	}
	
	public void loadWrapper(String[] specificParameters) throws SimException
	{
		if(specificParameters.length == this.getNumberOfSpecificParameters())
		{
			try
			{
				this.delay = Double.valueOf(specificParameters[0]).doubleValue();
				this.busSize = Integer.valueOf(specificParameters[1]).intValue();
				this.carryin = Boolean.valueOf(specificParameters[2]).booleanValue();
			}
			catch(NumberFormatException e)
			{
				throw (new SimException("incorrect parameter type"));
			}
		}
		else
			throw (new SimException("incorrect number of parameters"));
	}
	
	public int getNumberOfSpecificParameters()
	{
		return 3;
	}
	
/* ==================================================================
	Popup Part
	================================================================= */
	protected int oldBusSize = 0;
	
	public boolean hasProperties()
	{
		return true;
	}
	
	public Component getPropertyWindow()
	{
		return (new Function2fat1thinIn1fat1thinOutProperty(this.busSize, this.delay, this.carryin));
	}
		
	public void respondToChanges(Component property)
	{
		this.delay = ((Function2fat1thinIn1fat1thinOutProperty)property).getDelay();
		
		if(this.oldBusSize == 0)
			this.oldBusSize = this.busSize;
		this.carryin = ((Function2fat1thinIn1fat1thinOutProperty)property).getCarryIn();
		
		CentralPanel.ACTIVE_GRID.eraseComponent(this);
		this.setBusSize(((Function2fat1thinIn1fat1thinOutProperty)property).getBusSize());
		CentralPanel.ACTIVE_GRID.paintComponent(this);
		
	}
	
	public void restoreOriginalProperties()
	{
		if(this.oldBusSize != 0)
		{
			this.setBusSize(this.oldBusSize);
			this.oldBusSize = 0;
		}
	}
}
