package sim.lib.functions;

import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import sim.*;
import sim.engine.*;

import sim.lib.wires.Junction;

public class Minus extends Function2in1fat1thinOut
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/functions/SubtractUnitIcon.gif");
	
	public Image getIcon()
	{
		return Minus.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "Subtract unit";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public Minus()
	{
		super();
	}
	
	protected Wrapper getCopy()
	{
		Minus result = new Minus();
		result.changeDelay(this.delay);
		result.setBusSize(this.busSize);
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int tripple = 3 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawLine(gridGap + increment, gridGap, 10 * increment, gridGap);
			g.drawLine(14 * increment, gridGap, 19 * increment, gridGap);
			
			g.drawLine(10 * increment, gridGap, tripple, 6 * increment);
			g.drawLine(14 * increment, gridGap, tripple, 6 * increment);
			
			g.drawLine(gridGap + increment, tripple, 19 * increment, tripple);
			
			g.drawLine(gridGap + increment, gridGap, gridGap + increment, tripple);
			g.drawLine(19 * increment, gridGap, 19 * increment, tripple);
			
			
			g.drawLine(0, 2 * gridGap, gridGap + increment, 2 * gridGap);
			g.fillRect(2 * gridGap - 1, 0, 3, gridGap);
			g.fillRect(4 * gridGap - 1, 0, 3, gridGap);
			g.fillRect(tripple - 1, tripple, 3, gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("_", tripple - fm.stringWidth("_") / 2, 2 * gridGap);
		}
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		int loop;
		int x = 0;
		int y = 0;
		int base = 1;
		boolean foundUndefined = false;
		String bitValue;
		
		for(loop = 0; (loop < this.busSize) && (!foundUndefined); loop++)
		{
			if(currentInputs[loop].isUndefined() || currentInputs[loop + this.busSize].isUndefined())
				foundUndefined = true;
			else
			{
				if(currentInputs[loop].getValue())
					x = x + base;
				
				if(currentInputs[loop + this.busSize].getValue())
					y = y + base;
			}
			
			base = 2 * base;
		}
		
		if(foundUndefined)
		{
			for(loop = 0; loop <= this.busSize;loop++)
				peer.setOutputPinUndefined(loop, time);
		}
		else
		{
			x = x - y;
			
			peer.setOutputPinValue(this.busSize, x < 0, time);
			
			x = (int)Math.pow(2, this.busSize) + x;
			
			bitValue = Integer.toBinaryString(x);
			
			for(loop = bitValue.length(); loop < this.busSize; loop ++)
				bitValue = "0" + bitValue;
			
			for(loop = 0; loop < this.busSize; loop++)
				peer.setOutputPinValue(loop, bitValue.charAt(bitValue.length() - 1 - loop) == '1', time);
		}
	}
	
	public void writeVHDLEntity(String path) {
		VHDLGeneral.write2busto1busFunction(path, getVHDLName(), this.busSize, "f <= std_logic_vector(to_signed(to_integer(unsigned(x)) - to_integer(unsigned(y)),f'length));", false, true);
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				writer.write("use IEEE.numeric_std.all;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port( x0: in std_logic;\n");
				for(int x = 1; x < busSize; x++){
					writer.write("x" + x + ": in std_logic;\n");
				}
				
				for(int x = 0; x < busSize; x++){
					writer.write("y" + x + ": in std_logic;\n");
				}
				
				for(int x = 0; x < busSize; x++){
					writer.write("F" + x + ": out std_logic");
					if(x != (busSize-1))
						writer.write(";\n");
				}
				
				writer.write(";\ncout: out std_logic\n");
				
				writer.write(");\n");
				
				writer.write("end "+ this.getVHDLName() + ";\n\n");
				
				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("signal c: std_logic_vector(" + busSize + " downto 1);\n");
				writer.write("begin\n");

				writer.write("f0 <= 'Z' when x0 = 'Z' or y0 = 'Z' else\n");
				writer.write("      '0' when x0 = y0 else '1';\n"); 
				writer.write("c(1) <= 'Z' when (x0 = 'Z' or y0 = 'Z') else\n");
				writer.write("        '1' when x0 = '0' and y0 = '1' else '0';\n");
				for(int x = 1; x < busSize; x++) {
					writer.write("f" + x + " <= 'Z' when x" + x + " = 'Z' or y" + x + " = 'Z' or c(" + x + ") ='Z' else\n");
					writer.write("      '0' when (x" + x + " = '0' and y" + x + " = '0' and c(" + x + ") ='0') or (x" + x + " = '1' and y" + x + " = '1' and c(" + x + ") = '0') or (x" + x + " = '1' and y" + x + " = '0' and c(" + x + ") = '1') or (x" + x + " = '0' and y" + x + " = '1' and c(" + x + ") = '1') else '1';\n");
					writer.write("c(" + (x+1) + ") <= 'Z' when (x" + x + " = 'Z' and y" + x + " = 'Z' and c(" + x + ") ='0') or (x" + x + " = 'Z' and y" + x + " = '0' and c(" + x + ") = 'Z') or (x" + x + " = '0' and y" + x + " = 'Z' and c(" + x + ") = 'Z') or (x" + x + " = 'Z' and y" + x + " = 'Z' and c(" + x + ") = 'Z') else\n");
					writer.write("        '1' when (x" + x + " = '0' and y" + x + " = '1' and c(" + x + ") ='0') or (x" + x + " = '0' and y" + x + " = '1' and c(" + x + ") = '1') or (x" + x + " = '0' and y" + x + " = '0' and c(" + x + ") = '1') or (x" + x + " = '1' and y" + x + " = '1' and c(" + x + ") = '1') else '0';\n");
				}
				
				writer.write("cout <= c("+busSize+");\n");
				//writer.write(";\n");
				writer.write("end behav;\n");
				
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
	
	public String getVHDLName(){
		return "MINUSbus_ent_" + this.busSize;
	}
}
