package sim.lib.functions;

import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import sim.*;
import sim.engine.*;

import sim.lib.wires.Junction;

public class Negate extends Function1in1fatOut
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/functions/NegateIcon.gif");
	
	public Image getIcon()
	{
		return Negate.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "Negate";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public Negate()
	{
		super();
	}
	
	protected Wrapper getCopy()
	{
		Negate result = new Negate();
		result.changeDelay(this.delay);
		result.setBusSize(this.busSize);
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			
			g.setColor(this.brush);
			
			g.drawRect(0, gridGap, 2 * gridGap - 1, 2* gridGap - 1);
			
			g.fillRect(gridGap - 1, 0, 3, gridGap);
			g.fillRect(gridGap - 1, 3 * gridGap, 3, gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("_", gridGap - fm.stringWidth("_") / 2, 2 * gridGap);//10 * increment);
		}
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		int loop;
		int value = 0;
		int base = 1;
		boolean foundUndefined = false;
		String bitValue;
		
		for(loop = 0; (loop < this.busSize) && (!foundUndefined); loop++)
		{
			if(currentInputs[loop].isUndefined())
				foundUndefined = true;
			else if(currentInputs[loop].getValue())
				value = value + base;
			
			base = 2 * base;
		}
		
		if(foundUndefined)
		{
			for(loop = 0; loop < this.busSize;loop++)
				peer.setOutputPinUndefined(loop, time);
		}
		else
		{
			value = (int)Math.pow(2, this.busSize) - value;
			
			bitValue = Integer.toBinaryString(value);
			
			for(loop = bitValue.length(); loop <  this.busSize; loop ++)
				bitValue = "0" + bitValue;
			
			for(loop = 0; loop < this.busSize; loop++)
				peer.setOutputPinValue(loop, bitValue.charAt(bitValue.length() - 1 - loop) == '1', time);
		}
	}
	
	@Override
	public void writeVHDLEntity(String path) {
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				writer.write("use IEEE.numeric_std.all;\n");
				writer.write("use ieee.std_logic_unsigned.all;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port( x0: in std_logic;\n");
				for(int x = 1; x < busSize; x++){
					writer.write("x" + x + ": in std_logic;\n");
				}
				
				for(int x = 0; x < busSize; x++){
					writer.write("F" + x + ": out std_logic");
					if (x == busSize-1){
						writer.write("\n");
					}else {
						writer.write(";\n");
					}
				}
				
				writer.write(");\n");
				
				writer.write("end "+ this.getVHDLName() + ";\n\n");
				
				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("signal x: std_logic_vector(" +(busSize-1) + " downto 0);\n");
				writer.write("signal f: std_logic_vector(" +busSize + " downto 0);\n");
				writer.write("begin\n");
				
				for(int x = 0; x < busSize; x++){
					writer.write("x(" + x + ") <= x" + x + ";\n");
				}
				writer.write("f <= std_logic_vector(to_unsigned(" + (int)(Math.pow(2,busSize)) + ",f'length)) - x;\n");
				for(int x = 0; x < busSize; x++){
					writer.write("f" + x + " <= f(" + x + ");\n");
				}
				writer.write("end behav;\n");
				
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	@Override
	public String getVHDLName() {
		return "NEGATEbus_ent_" + this.busSize;
	}
}