package sim.lib.functions;

import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import sim.*;
import sim.engine.*;

import sim.lib.wires.Junction;

public class ShiftRight extends Function1fat1thinIn1fatOut
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/functions/ShiftRightIcon.gif");
	
	public Image getIcon()
	{
		return ShiftRight.ICON;
	}
	
	public String getBubbleHelp()
	{
		return "Shift right";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public ShiftRight()
	{
		super();
	}
	
	protected Wrapper getCopy()
	{
		ShiftRight result = new ShiftRight();
		result.changeDelay(this.delay);
		result.setBusSize(this.busSize);
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawRect(gridGap, gridGap, twice - 1, twice);
			
			g.drawLine(0, 2 * gridGap, gridGap, twice);
			g.fillRect(twice - 1, 0, 3, gridGap);
			g.fillRect(twice - 1, 3 * gridGap, 3, gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString(">>", twice - fm.stringWidth(">>") / 2, twice + fm.getAscent() / 2);
		}
	}
	
/* ==================================================================
	Simulation part
	================================================================= */
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		for(int loop = 0; loop < this.busSize; loop++)
		{
			if(currentInputs[loop + 1].isUndefined())
				peer.setOutputPinUndefined(loop, time);
			else
				peer.setOutputPinValue(loop, currentInputs[loop + 1].getValue(), time);
		}
		
	}
	
	@Override
	public void writeVHDLEntity(String path) {
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port( x0: in std_logic;\n");
				for(int x = 1; x < busSize; x++){
					writer.write("x" + x + ": in std_logic;\n");
				}
				writer.write("C: in std_logic");
				for(int x = 0; x < busSize; x++){
					writer.write(";\nF" + x + ": out std_logic");
				}
				writer.write("\n");
				writer.write(");\n");
				writer.write("end "+ this.getVHDLName() + ";\n\n");
				
				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("begin\n");
				for(int x = 1; x < busSize; x++){
					writer.write("f" + (x - 1) + " <= x" + x + ";\n");
				}
				writer.write("f" + (busSize - 1) + " <= C;\n");
				writer.write("end behav;\n");
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public String getVHDLName() {
		return "SHIFTRIGHTbus_ent_" + this.busSize;
	}
}