package sim.lib.memory;

import java.awt.*;
import java.awt.event.*;
import sim.util.*;
import sim.*;
import java.io.*;

public class RamProperties extends Container implements ActionListener, FocusListener {

	private DialogButton browse = new DialogButton("Browse...");
	private TextField editComponentName = new TextField();
	private TextField editFileName = new TextField();
	private TextField editBusSize = new TextField(5);
	private TextField editAddressSize = new TextField(5);
	private TextField editAddressLoadDelay = new TextField(5);
	private TextField editEnableDelay = new TextField(5);
	private TextField editMinWriteCycle = new TextField(5);
	private TextField editWriteToRead = new TextField(5);
	
	private Label componentName = new Label("Name");
	private Label content = new Label("Memory Content");
	private Label busSize = new Label("Bus Sizes");
	private Label delays = new Label("Propagation Delays");
	private String oldComponentName;
	private String oldName;
	private int oldBusSize;
	private int oldAddressSize;
	private double oldAddressLoadDelay;
	private double oldEnableDelay;
	private double oldMinWriteCycle;
	private double oldWriteToRead;
	
	private ScrollPane scroller = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);

	public RamProperties(String file, String compName, int bus, int address, double addressLoadTime, double enableDelay, double minWriteCycle, double writeToReadDelay) {
		super();
		this.setLayout(new GridBagLayout());

		if (MainWindow.MASTER != null) {
			this.editFileName.setEnabled(false);
			this.browse.setEnabled(false);
		}

		this.oldComponentName = compName;
		this.oldName = file;
		this.oldBusSize = bus;
		this.oldAddressSize = address;
		this.oldAddressLoadDelay = addressLoadTime;
		this.oldEnableDelay = enableDelay;
		this.oldMinWriteCycle = minWriteCycle;
		this.oldWriteToRead = writeToReadDelay;

		this.editComponentName.setText(this.oldComponentName);
		this.editFileName.setText(this.oldName);
		this.editBusSize.setText(Integer.toString(this.oldBusSize));
		this.editAddressSize.setText(Integer.toString(this.oldAddressSize));
		this.editAddressLoadDelay.setText(Double.toString(this.oldAddressLoadDelay));
		this.editEnableDelay.setText(Double.toString(this.oldEnableDelay));
		this.editMinWriteCycle.setText(Double.toString(this.oldMinWriteCycle));
		this.editWriteToRead.setText(Double.toString(this.oldWriteToRead));

		this.editComponentName.addActionListener(this);
		this.browse.addActionListener(this);
		this.editFileName.addActionListener(this);
		this.editBusSize.addActionListener(this);
		this.editAddressSize.addActionListener(this);
		this.editAddressLoadDelay.addActionListener(this);
		this.editEnableDelay.addActionListener(this);
		this.editMinWriteCycle.addActionListener(this);
		this.editWriteToRead.addActionListener(this);

		this.editComponentName.addFocusListener(this);
		this.editFileName.addFocusListener(this);
		this.editBusSize.addFocusListener(this);
		this.editAddressSize.addFocusListener(this);
		this.editAddressLoadDelay.addFocusListener(this);
		this.editEnableDelay.addFocusListener(this);
		this.editMinWriteCycle.addFocusListener(this);
		this.editWriteToRead.addFocusListener(this);

		// component name
		Panel big = new Panel(new BorderLayout(0, 15));
		Panel p = new Panel(new BorderLayout());
		GridBagConstraints c = new GridBagConstraints();

		p.add(this.componentName, BorderLayout.WEST);
		p.add(new SimSeparator(), BorderLayout.CENTER);
		big.add(p, BorderLayout.NORTH);

		p = new Panel(new GridBagLayout());

		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 1;
		c.gridheight = 1;
		c.weightx = 1;
		c.weighty = 0;
		c.anchor = GridBagConstraints.WEST;
		c.fill = GridBagConstraints.HORIZONTAL;

		p.add(this.editComponentName, c);
		big.add(p, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1;
		c.insets = new Insets(0, 0, 15, 0);
		c.fill = GridBagConstraints.HORIZONTAL;

		this.add(big, c);

		// content
		big = new Panel(new BorderLayout(0, 15));
		p = new Panel(new BorderLayout());

		p.add(this.content, BorderLayout.WEST);
		p.add(new SimSeparator(), BorderLayout.CENTER);
		big.add(p, BorderLayout.NORTH);

//		p = new Panel(new GridBagLayout());
//		c = new GridBagConstraints();
//
//		c.gridx = 0;
//		c.gridy = 1;
//		c.gridwidth = 1;
//		c.gridheight = 1;
//		c.weightx = 1;
//		c.weighty = 0;
//		c.anchor = GridBagConstraints.WEST;
//		c.fill = GridBagConstraints.HORIZONTAL;
//		p.add(this.editFileName, c);
//
//		c.gridx = 1;
//		c.weightx = 0;
//		c.fill = GridBagConstraints.NONE;
//		c.insets = new Insets(0, 10, 0, 0);
//		p.add(this.browse, c);
//		big.add(p, BorderLayout.CENTER);

		Label memPath = new Label(this.oldName);
		big.add(memPath, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 1;
		c.weightx = 1;
		c.insets = new Insets(0, 0, 15, 0);
		c.fill = GridBagConstraints.HORIZONTAL;

		this.add(big, c);


		// bus size
		big = new Panel(new BorderLayout(0, 15));
		p = new Panel(new BorderLayout());

		p.add(this.busSize, BorderLayout.WEST);
		p.add(new SimSeparator(), BorderLayout.CENTER);
		big.add(p, BorderLayout.NORTH);
		p = new Panel(new GridBagLayout());
		Panel p2 = new Panel(new FlowLayout(FlowLayout.LEFT, 0, 0));

		p2.add(new Label("Data"));
		p2.add(this.editBusSize);

		c.fill = GridBagConstraints.NONE;
		c.insets = new Insets(0, 0, 0, 0);

		p.add(p2, c);
		p2 = new Panel(new FlowLayout(FlowLayout.RIGHT, 0, 0));

		p2.add(new Label("Address"));
		p2.add(this.editAddressSize);

		c.anchor = GridBagConstraints.EAST;
		c.gridx = 1;
		p.add(p2, c);

		big.add(p, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 2;
		c.insets = new Insets(0, 0, 15, 0);
		c.fill = GridBagConstraints.HORIZONTAL;
		this.add(big, c);

		// simulation
		big = new Panel(new BorderLayout(0, 15));
		p = new Panel(new BorderLayout());

		p.add(this.delays, BorderLayout.WEST);
		p.add(new SimSeparator(), BorderLayout.CENTER);
		big.add(p, BorderLayout.NORTH);

		p = new Panel(new GridBagLayout());
		c.insets = new Insets(2, 5, 2, 0);

		c.gridy = 0;
		p.add(new Label("address access delay"), c);
		c.gridx = 1;
		p.add(this.editAddressLoadDelay, c);

		c.gridy = 1;
		c.gridx = 0;
		p.add(new Label("chip/output enable delay"), c);
		c.gridx = 1;
		p.add(this.editEnableDelay, c);

		c.gridy = 2;
		c.gridx = 0;
		p.add(new Label("minimum write time"), c);
		c.gridx = 1;
		p.add(this.editMinWriteCycle, c);

		c.gridy = 3;
		c.gridx = 0;
		p.add(new Label("return to read mode after write"), c);
		c.gridx = 1;
		p.add(this.editWriteToRead, c);

		this.scroller.getVAdjustable().setBlockIncrement(50);
		this.scroller.getVAdjustable().setUnitIncrement(25);
		this.scroller.add(p);

		big.add(this.scroller, BorderLayout.CENTER);

		c.gridx = 0;
		c.gridy = 3;
		c.insets = new Insets(0, 0, 0, 0);
		this.add(big, c);
	}

	public void addNotify() {
		super.addNotify();
		this.setSize(290, this.editBusSize.getPreferredSize().height * 5 + this.delays.getPreferredSize().height * 3 + 200);

		this.scroller.validate();
	}

	public void actionPerformed(ActionEvent e) {
		Component source = (Component) e.getSource();

		if (source == this.editFileName) {
			this.getFileName();
		} else if (source == this.editBusSize) {
			this.getBusSize();
		} else if (source == this.editAddressSize) {
			this.getAddressSize();
		} else if (source == this.editAddressLoadDelay) {
			this.getAddressBeforeRead();
		} else if (source == this.editEnableDelay) {
			this.getEnableDelay();
		} else if (source == this.editMinWriteCycle) {
			this.getMinimumWriteCycle();
		} else if (source == this.editWriteToRead) {
			this.getReturnToRead();
		} else if (source == this.browse) {
			SaveLoadShortcut.FILEDIALOG.setTitle("Open");
			//SaveLoadShortcut.FILEDIALOG.setMode(FileDialog.LOAD);
			SaveLoadShortcut.FILEDIALOG.setFile("*.mem");
			SaveLoadShortcut.FILEDIALOG.setVisible(true);

			String inFile = SaveLoadShortcut.FILEDIALOG.getFile();

			if (inFile != null) {
				this.oldName = SaveLoadShortcut.FILEDIALOG.getDirectory() + inFile;
				this.editFileName.setText(this.oldName);
			}
		}
	}

	public void focusGained(FocusEvent e) {
	}

	public void focusLost(FocusEvent e) {
		TextField source = (TextField) e.getSource();

		if (source == this.editFileName) {
			this.getFileName();
		} else if (source == this.editBusSize) {
			this.getBusSize();
		} else if (source == this.editAddressSize) {
			this.getAddressSize();
		} else if (source == this.editAddressLoadDelay) {
			this.getAddressBeforeRead();
		} else if (source == this.editEnableDelay) {
			this.getEnableDelay();
		} else if (source == this.editMinWriteCycle) {
			this.getMinimumWriteCycle();
		} else if (source == this.editWriteToRead) {
			this.getReturnToRead();
		}

		source.setText(source.getText());
	}

	public int getBusSize() {
		int newBus;

		try {
			newBus = Integer.valueOf(this.editBusSize.getText()).intValue();

			if ((newBus > 1) && (newBus <= 32)) {
				this.oldBusSize = newBus;
			} else {
				this.editBusSize.setText(Integer.toString(this.oldBusSize));
			}
		} catch (NumberFormatException nfe) {
			this.editBusSize.setText(Integer.toString(this.oldBusSize));
		}

		return this.oldBusSize;
	}

	public int getAddressSize() {
		int newAddress;

		try {
			newAddress = Integer.valueOf(this.editAddressSize.getText()).intValue();

			if ((newAddress > 1) && (newAddress <= 8)) {
				this.oldAddressSize = newAddress;
			} else {
				this.editAddressSize.setText(Integer.toString(this.oldAddressSize));
			}
		} catch (NumberFormatException nfe) {
			this.editAddressSize.setText(Integer.toString(this.oldAddressSize));
		}

		return this.oldAddressSize;
	}

	public String getFileName() {
		String newName = this.editFileName.getText();

		if (this.browse.isEnabled()) {
			try {
				FileWriter f = new FileWriter(newName, true);
				f.close();

				this.oldName = newName;
			} catch (IOException ex) {
				this.editFileName.setText(this.oldName);
			}
		}

		return this.oldName;
	}

	public double getAddressBeforeRead() {
		double newValue;

		try {
			newValue = Double.valueOf(this.editAddressLoadDelay.getText()).doubleValue();

			if (newValue >= 0) {
				this.oldAddressLoadDelay = newValue;
			} else {
				this.editAddressLoadDelay.setText(Double.toString(this.oldAddressLoadDelay));
			}
		} catch (NumberFormatException nfe) {
			this.editAddressLoadDelay.setText(Double.toString(this.oldAddressLoadDelay));
		}

		return this.oldAddressLoadDelay;
	}

	public double getEnableDelay() {
		double newValue;

		try {
			newValue = Double.valueOf(this.editEnableDelay.getText()).doubleValue();

			if (newValue >= 0) {
				this.oldEnableDelay = newValue;
			} else {
				this.editEnableDelay.setText(Double.toString(this.oldEnableDelay));
			}
		} catch (NumberFormatException nfe) {
			this.editEnableDelay.setText(Double.toString(this.oldEnableDelay));
		}

		return this.oldEnableDelay;
	}

	public double getMinimumWriteCycle() {
		double newValue;

		try {
			newValue = Double.valueOf(this.editMinWriteCycle.getText()).doubleValue();

			if (newValue >= 0) {
				this.oldMinWriteCycle = newValue;
			} else {
				this.editMinWriteCycle.setText(Double.toString(this.oldMinWriteCycle));
			}
		} catch (NumberFormatException nfe) {
			this.editMinWriteCycle.setText(Double.toString(this.oldMinWriteCycle));
		}

		return this.oldMinWriteCycle;
	}

	public double getReturnToRead() {
		double newValue;

		try {
			newValue = Double.valueOf(this.editWriteToRead.getText()).doubleValue();

			if (newValue >= 0) {
				this.oldWriteToRead = newValue;
			} else {
				this.editWriteToRead.setText(Double.toString(this.oldWriteToRead));
			}
		} catch (NumberFormatException nfe) {
			this.editWriteToRead.setText(Double.toString(this.oldWriteToRead));
		}

		return this.oldWriteToRead;
	}

	public String getComponentName() {
		String newName;

		try {
			newName = this.editComponentName.getText();

			this.oldComponentName = newName;
		} catch (NumberFormatException nfe) {
		}

		this.editComponentName.setText(this.oldComponentName);

		return this.oldComponentName;
	}

	public Dimension getPreferredSize() {
		return this.getSize();
	}

	public Dimension getMinimumSize() {
		return this.getSize();
	}

	public Dimension getMaximumSize() {
		return this.getSize();
	}
}
