package sim.lib.others;

import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import sim.*;
import sim.engine.*;
import sim.lib.EditDelay;
import sim.lib.wires.Junction;

public class HalfAdder extends WrapperPainted implements EngineModule
{
/* ==================================================================
	Creation Part
	================================================================= */
	private static Image ICON = GuiFileLink.getImage("sim/lib/others/HalfAdderIcon.gif");
	
	public Image getIcon()
	{
		return HalfAdder.ICON;
	}
	
	public Wrapper createWrapper()
	{
		return this.getCopy();
	}
	
	public Wrapper createWrapper(Point gridPosition)
	{
		HalfAdder result = this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}
	
	public String getBubbleHelp()
	{
		return "Half Adder";
	}
	
/* ==================================================================
	GUI part
	================================================================= */
	public HalfAdder()
	{
		super();
	}
	
	protected HalfAdder getCopy()
	{
		HalfAdder result = new HalfAdder();
		
		result.changeDelay(this.delay);
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}
	
	public void initializeGridSize()
	{
		this.setGridSize(5, 4);
	}
	
	public void paint(Graphics g)
	{
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int tripple = 3 * gridGap;

			g.setColor(this.brush);

			g.drawLine(gridGap + increment, gridGap, 19 * increment, gridGap);
			g.drawLine(gridGap + increment, tripple, 19 * increment, tripple);
			g.drawLine(gridGap + increment, gridGap, gridGap + increment, tripple);
			g.drawLine(19 * increment, gridGap, 19 * increment, tripple);

			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			g.drawLine(2 * gridGap, 0, 2 * gridGap, gridGap);
			g.drawString("X", gridGap + increment, 3 * increment);
			
			g.drawLine(4 * gridGap, 0, 4 * gridGap, gridGap);
			g.drawString("Y", 4 * gridGap + increment, 3 * increment);

			g.drawLine(0, 2 * gridGap, gridGap + increment, 2 * gridGap);
			g.drawString("C", 2 * increment, 2 * gridGap);
			
			g.drawLine(tripple, tripple, tripple, 4 * gridGap);
			g.drawString("S", tripple + increment, tripple + 3 * increment);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 5 * increment));
			g.drawString("H A", 2 * gridGap, 2 * gridGap + 2 * increment);
		}
	}
	
/* ==================================================================
	Maintanance Part
	================================================================= */
	private Junction inputX = null;
	private Junction inputY = null;
	private Junction outputS = null;
	private Junction outputC = null;
	
	public boolean canDrop()
	{
		boolean result = Wrapper.canDropJuncion(this.gridLocation.x + 3, this.gridLocation.y + 4, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 2, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y, 1);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 4, this.gridLocation.y, 1);
		
		return result;
	}
		
	public void droped()
	{
		this.outputS = Wrapper.setPinAt(this.gridLocation.x + 3, this.gridLocation.y + 4, 1);
		this.outputC = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 2, 1);
		this.inputX = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y, 1);
		this.inputY = Wrapper.setPinAt(this.gridLocation.x + 4, this.gridLocation.y, 1);
		
		this.changeColor(Color.black);
	}
	
	public void selected()
	{
		this.inputX.removePin();
		this.inputY.removePin();
		this.outputC.removePin();
		this.outputS.removePin();
		
		this.changeColor(Color.green);
	}
	
	public void checkAfterSelected()
	{
		Wrapper.checkPin(this.inputX);
		Wrapper.checkPin(this.inputY);
		Wrapper.checkPin(this.outputC);
		Wrapper.checkPin(this.outputS);
	}
/* ==================================================================
	Simulation part
	================================================================= */
	protected double delay = 1;
	
	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer)
	{
		double time = this.delay + currentTime;
		
		if(currentInputs[0].isUndefined() || currentInputs[1].isUndefined())
		{
			peer.setOutputPinUndefined(0, time);
			peer.setOutputPinUndefined(1, time);
			
		}
		else
		{
			if(currentInputs[0].getValue() && currentInputs[1].getValue())
			{
				peer.setOutputPinValue(0, false, time);
				peer.setOutputPinValue(1, true, time);
			}
			else if(currentInputs[0].getValue() || currentInputs[1].getValue())
			{
				peer.setOutputPinValue(0, true, time);
				peer.setOutputPinValue(1, false, time);
			}
			else
			{
				peer.setOutputPinValue(0, false, time);
				peer.setOutputPinValue(1, false, time);
			}
		}
	}
	
	public void createEnginePeer(EnginePeerList epl)
	{
		EnginePeer ep = new EnginePeer(2, 2, this);
		
		ep.setInputPin(0, this.inputX.getNodes().getItemAt(0));
		ep.setInputPin(1, this.inputY.getNodes().getItemAt(0));
		
		ep.setOutputPin(0, this.outputS.getNodes().getItemAt(0));
		ep.setOutputPin(1, this.outputC.getNodes().getItemAt(0));
		
		epl.insertItem(ep);
	}
	
	public void reset()
	{
	}
	
	public Wrapper getParentWrapper()
	{
		return this;
	}
	
	public double getDelay()
	{
		return this.delay;
	}
	
	public void changeDelay(double newValue)
	{
		this.delay = newValue;
	}
	
/* ==================================================================
	Storage Part
	================================================================= */
	public String getSpecificParameters()
	{
		return (Double.toString(this.delay) + Wrapper.SEPARATOR);
	}
	
	public void loadWrapper(String[] specificParameters) throws SimException
	{
		if(specificParameters.length == this.getNumberOfSpecificParameters())
		{
			try
			{
				this.delay = Double.valueOf(specificParameters[0]).doubleValue();
			}
			catch(NumberFormatException e)
			{
				throw (new SimException("incorrect parameter type"));
			}
		}
		else
			throw (new SimException("incorrect number of parameters"));
	}
	
	public int getNumberOfSpecificParameters()
	{
		return 1;
	}
	
/* ==================================================================
	Popup Part
	================================================================= */
	public boolean hasProperties()
	{
		return true;
	}
	
	public Component getPropertyWindow()
	{
		return (new EditDelay(this.delay));
	}
		
	public void respondToChanges(Component property)
	{
		this.delay = ((EditDelay)property).getDelay();
	}

	@Override
	public void writeVHDLEntity(String path) {
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				writer.write("use ieee.numeric_std.all;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port( x: in std_logic;\n");
				writer.write("y : in std_logic;\n");
				writer.write("cout : out std_logic;\n");
				writer.write("sum : out std_logic);\n");
				writer.write("end "+ this.getVHDLName() + ";\n\n");
				
				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("begin\n");
				writer.write("sum <= 'Z' when x = 'Z' or y = 'Z' else\n");
				writer.write("       '1' when (x = '1' and y = '0' ) or (x = '0' and y = '1') else '0';\n");
				writer.write("cout <= 'Z' when (x = 'Z' and y = 'Z') else\n");
				writer.write("        '1' when (x = '1' and y = '1') else '0';\n");
				writer.write("end behav;\n");
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	@Override
	public String getVHDLName() {
		// TODO Auto-generated method stub
		return null;
	}
}
