package sim.lib.others;

import java.awt.*;
import java.awt.geom.AffineTransform;
import java.io.*;
//import java.nio.file.*;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.swing.*;

import sim.*;
import sim.lib.*;
import sim.engine.*;
import sim.lib.outputs.BusLed;
import sim.lib.wires.Junction;
import sim.lib.wires.SplitterModule;
import sim.lib.wires.Wire;

public class Module extends WrapperPainted implements EngineModule {
	/* ==================================================================
	 Creation Part
	 ================================================================= */

	private static Image ICON = GuiFileLink.getImage("sim/lib/others/Module.gif");

	public Image getIcon() {
		return Module.ICON;
	}

	public Wrapper createWrapper() {
		return this.getCopy();
	}

	public Wrapper createWrapper(Point gridPosition) {
		Module result = this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}

	public String getBubbleHelp() {
		return "Module";
	}
	/* ==================================================================
	 GUI part
	 ================================================================= */
	private String value = null;
	private int busSize;
	private int valueLenght;
	private String modpath = null;
	private int numPin;
	private boolean pin99 = false;
	private String[] labels = new String[100];
	private String modLabel = null;
	public static String lastPath;
	//private static String errorPath;
	private static boolean loadError;
	private int[] busSizes = new int[100];
	private static loadedModule[] loaded = new loadedModule[50];
	private String regName;
	private Object regOut = null;
	private String initRegName; //for setting intial value of a register
	private String initRegValue;
	private String[] regList = new String[50];
	private boolean colour = false;
	private int ModID = -1;
	public static int nextID = 0;
	public static Map<String,Integer> CustomIDs = new HashMap<String,Integer>();
	private boolean incremental;
	private int startId;
	private int setId;

	public Module() {
		super();

		this.setBusSize(8);

		//this.setPath(lastPath);
	}

	public Module getCopy() {
		Module result = new Module();
		result.setPath(lastPath);
		result.setRName(this.regName);
		result.defaultLabel();
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());

		//result.setBusSize(this.busSize);

		return result;
	}

	public void defaultLabel() {
		if (this.modpath != null) {
			String parts[] = this.modpath.split("\\\\");
			String name = parts[parts.length - 1];
			name = name.substring(0, name.length() - 4);
			this.modLabel = name;
		}
	}

	public String getLabel() {
		return this.modLabel;
	}

	public void setLabel(String label) {
		this.modLabel = label;
	}
	
	public void setModID(int id){
		this.ModID = id;
	}

	public void setRName(String reg) {
		this.regName = reg;
	}

	public void setRegOut(Object r) {
		this.regOut = r;
	}
	
	public int getModID(int id){
		return this.ModID;
	}

	public String getPath() {
		return this.modpath;
	}

	public void setPath(String file) {
		this.modpath = file;

		//LOAD numPin = number of pin components from file
		//get names for pins into string array
		this.numPin = 0;
		Grid g = new Grid();
		String[] basics, specifics;
		String className, componentName, readIn;
		BufferedReader inStream;
		Wrapper created;
		int z = 0;

		try {
			inStream = new BufferedReader(new FileReader(file));
			readIn = SaveLoadShortcut.GUI_FILE_LINK.extractParameter(inStream);
			SaveLoadShortcut.GUI_FILE_LINK.readBlank(inStream);
			SaveLoadShortcut.GUI_FILE_LINK.extractParameters(g.getNumberOfParameters(), inStream);
			SaveLoadShortcut.GUI_FILE_LINK.readBlank(inStream);
			int size = Integer.valueOf(SaveLoadShortcut.GUI_FILE_LINK.extractParameter(inStream)).intValue();
			for (int index = 0; index < size; index++) {
				SaveLoadShortcut.GUI_FILE_LINK.readBlank(inStream);
				className = SaveLoadShortcut.GUI_FILE_LINK.extractParameter(inStream);
				componentName = SaveLoadShortcut.GUI_FILE_LINK.extractParameter(inStream);
				created = SaveLoadShortcut.GUI_FILE_LINK.getWrapper(className);

				basics = SaveLoadShortcut.GUI_FILE_LINK.extractParameters(created.getNumberOfBasicParameters(), inStream);
				specifics = SaveLoadShortcut.GUI_FILE_LINK.extractParameters(created.getNumberOfSpecificParameters(), inStream);
				if (className.equals("sim.lib.outputs.Pin")) {
					if(Integer.valueOf(specifics[1]) == 99)
						this.pin99 = true;
					else
						this.numPin++;
					this.busSizes[Integer.valueOf(specifics[1]) - 1] = Integer.valueOf(specifics[0]); //get bus size for each pin
					
					this.labels[Integer.valueOf(specifics[1]) - 1] = specifics[2]; //get label string for each pin
					//TODO: DELETE THIS WHEN DONE TESTING
					//System.out.println((this.labels[Integer.valueOf(specifics[1]) - 1]) + " : " + (Integer.valueOf(specifics[1]) - 1));

				} else if (className.equals("sim.lib.memory.Register") || className.equals("sim.lib.memory.ShiftRegister")) {
					this.regList[z] = specifics[2];
					z++;
				}
			}
			lastPath = file;

		} catch (Exception e) {
			if (!loadError) {
				JOptionPane.showMessageDialog(null, "Error loading module file, please correct file paths in Control Center", "ERROR", JOptionPane.ERROR_MESSAGE);
				loadError = true;
			}
		}


	}

	public String[] getRegList() {
		return this.regList;
	}

	public void initializeGridSize() {
		this.setGridSize(6, 7);
	}

	public void setBusSize(int size) {
		this.busSize = size;

		String max = Integer.toHexString((int) Math.pow(2, this.busSize) - 1).toUpperCase();
		FontMetrics fm = this.getFontMetrics(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * Grid.SIZE / 4));

		//this.setGridSize(fm.stringWidth(max) / Grid.SIZE + 4, 4);
		//this.setGridSize(6,6);
		this.valueLenght = max.length();
	}

	public int getBusSize() {
		return this.busSize;
	}

	public void paintValue(Graphics g) {
		if (this.isVisible()) {
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int offset = 2 * gridGap;
			String val = "";
			if (this.regOut != null) {
				if (this.regOut instanceof sim.lib.memory.Register) {
					val = ((sim.lib.memory.Register)this.regOut).getValue();
				}else if (this.regOut instanceof sim.lib.memory.ShiftRegister) {
					val = ((sim.lib.memory.ShiftRegister)this.regOut).getValue();
				}
				g.setFont(new Font(Wrapper.FONT_NAME, Font.BOLD, gridGap));
			}
			if (this.colour) {
				int col = Integer.decode(val);
				g.setColor(new Color(col,col,col));
				g.fillRect(2 * gridGap + increment / 2, 2 * gridGap + increment / 2, offset - increment+1, offset - increment+1);
			}else{
				g.drawString(val, 2 * gridGap + increment + 1, gridGap * 2 + 5 * increment);
			}
		}
	}

	public void paint(Graphics g) {
		//System.out.println(this.labels[2]);
		// draw if visible
		if (this.isVisible()) {
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;

			//g.setColor(this.brush);

			int offset = 2 * gridGap;
			g.setColor(Color.WHITE);
			g.fillRect(2 * gridGap + increment / 2, 2 * gridGap + increment / 2, offset - increment, offset - increment);
			g.setColor(this.brush);

			g.drawRect(2 * gridGap, 2 * gridGap, offset, offset);
			g.drawRect(2 * gridGap + increment / 2, 2 * gridGap + increment / 2, offset - increment, offset - increment);

			//g.drawRect(gridGap + gridGap / 5, 2 * increment + gridGap / 5, offset - 2*gridGap/5, offset -2*gridGap/5);

			int start = 3 * gridGap;
			int end = 5 * gridGap;
			offset = 2 * gridGap;
			g.setFont(new Font(Wrapper.FONT_NAME, Font.BOLD, gridGap * 2 / 3));
			Font f = new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment);
			String name;
			if (modLabel == null) {
				String parts[] = this.modpath.split("\\\\");
				name = parts[parts.length - 1];
				name = name.substring(0, name.length() - 4);
				this.modLabel = name;
			} else {
				name = modLabel;
			}
			int col = 0;
			if (this.regOut != null) {
				if (this.regOut instanceof sim.lib.memory.Register) {
					name = ((sim.lib.memory.Register)this.regOut).getValue();
					if(name.equals("--"))
						col = 0;
					else
						col = Integer.decode("0x" + name);
				}else if (this.regOut instanceof sim.lib.memory.ShiftRegister) {
					name = ((sim.lib.memory.ShiftRegister)this.regOut).getValue();
					//col = ((sim.lib.memory.ShiftRegister)this.regOut).getHex();
					if(name.equals("--"))
						col = 0;
					else
						col = Integer.decode("0x" + name);
				}
				if (this.colour) {
					/*System.out.println(name);
					int col = Integer.decode("0x" + name);
					System.out.println(col + "- " + name);*/
					//int col = ((sim.lib.memory.Register)this.regOut).getHex();
					//System.out.println(col);
					g.setColor(new Color(col,col,col));
					g.fillRect(2 * gridGap + increment / 2, 2 * gridGap + increment / 2, offset-increment, offset-increment);
					name = "";
					g.setColor(Color.BLACK);
				}else{
					g.setFont(new Font(Wrapper.FONT_NAME, Font.BOLD, gridGap));
				}
			}
			g.drawString(name, 2 * gridGap + increment + 1, gridGap * 2 + 5 * increment);
			//g.drawString(this.modpath, 0,0);

			offset = offset + gridGap;

			//**********Draw ports based on number of pins in input file**********
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, gridGap * 2 / 3));
			if(this.pin99){
				g.drawLine(4 *gridGap, 2 * gridGap, 5 * gridGap, 1 * gridGap); //portH (DIAGONAL)
			}
			if (this.labels[0] != null) {
				g.drawString(this.labels[0].substring(0, Math.min(5, this.labels[0].length())), gridGap / 8, gridGap * 5 / 2 + increment); //GET PIN NAMES FROM INPUT FILE
				if(this.busSizes[0] > 1) {
					g.fillRect(0, 3 * gridGap - 1, 2 * gridGap, 3); //portA (LEFT)
				} else {
					g.fillRect(0, 3 * gridGap, 2 * gridGap, 1);
				}
			}
			if (this.labels[1] != null) {
				AffineTransform tmp = f.getTransform();
				AffineTransform at = new AffineTransform();
				at.rotate(-90 * java.lang.Math.PI / 180);
				Font tf = f.deriveFont(at);
				g.setFont(tf.deriveFont((float) (gridGap * 2 / 3)));
				g.drawString(this.labels[1].substring(0, Math.min(5, this.labels[1].length())), gridGap * 11 / 4, gridGap * 2 - gridGap / 8);

				if(this.busSizes[1] > 1) {
					g.fillRect(3 * gridGap - 1, 0, 3, 2 * gridGap); //portB (TOP)
				} else {
					g.fillRect(3 * gridGap, 0, 1, 2 * gridGap);
				}
				
				/*for(int i = 0; i < this.labels[1].length(); i++){
				 if (i > 10){
				 break;
				 }
				 g.drawString(this.labels[1].charAt(i)+"", gridGap*3/2, gridGap*2/3+i*gridGap/2);
				 }*/
			}
			if (this.labels[2] != null) {
				g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, gridGap * 2 / 3));
				
				g.drawString(this.labels[2].substring(0, Math.min(5, this.labels[2].length())), gridGap * 4 + gridGap / 8, gridGap * 5 / 2 + increment);
				
				if(this.busSizes[2] > 1) {
					g.fillRect(4 * gridGap, 3 * gridGap - 1, 2 * gridGap, 3); //portC (RIGHT)
				} else {
					g.fillRect(4 * gridGap, 3 * gridGap, 2 * gridGap, 1);
				}
				
			}
			if (this.labels[3] != null) {
				AffineTransform tmp = f.getTransform();
				AffineTransform at = new AffineTransform();
				at.rotate(-90 * java.lang.Math.PI / 180);
				Font tf = f.deriveFont(at);
				g.setFont(tf.deriveFont((float) (gridGap * 2 / 3)));
				g.drawString(this.labels[3].substring(0, Math.min(5, this.labels[3].length())), gridGap * 7 / 2 + gridGap / 8, gridGap * 6 - gridGap / 8);

				
				if(this.busSizes[3] > 1) {
					g.fillRect(3 * gridGap - 1, 4 * gridGap, 3, 2 * gridGap); //portD (BOTTOM)
				} else {
					g.fillRect(3 * gridGap, 4 * gridGap, 1, 2 * gridGap);
				}
			}
			if(this.labels[4] != null) {
				g.drawLine(gridGap, gridGap, 2 * gridGap, 2 * gridGap); //portE (TOP-LEFT DIAGONAL)
				if(this.busSizes[4] > 1) {
					g.drawLine(gridGap, gridGap - 1, 2 * gridGap, 2 * gridGap - 1); //portE (TOP-LEFT DIAGONAL)
					g.drawLine(gridGap, gridGap + 1, 2 * gridGap, 2 * gridGap + 1); //portE (TOP-LEFT DIAGONAL)
					g.drawLine(gridGap, gridGap - 2, 2 * gridGap, 2 * gridGap - 2); //portE (TOP-LEFT DIAGONAL)
					g.drawLine(gridGap, gridGap + 2, 2 * gridGap, 2 * gridGap + 2); //portE (TOP-LEFT DIAGONAL)
				}
			}
			if(this.labels[5] != null) {
				g.drawLine(4 *gridGap, 2 * gridGap, 5 * gridGap, 1 * gridGap); //portF (TOP-RIGHT DIAGONAL)
				if(this.busSizes[5] > 1) {
					g.drawLine(4 *gridGap, 2 * gridGap - 1, 5 * gridGap, 1 * gridGap - 1); //portF (TOP-RIGHT DIAGONAL)
					g.drawLine(4 *gridGap, 2 * gridGap + 1, 5 * gridGap, 1 * gridGap + 1); //portF (TOP-RIGHT DIAGONAL)
					g.drawLine(4 *gridGap, 2 * gridGap - 2, 5 * gridGap, 1 * gridGap - 2); //portF (TOP-RIGHT DIAGONAL)
					g.drawLine(4 *gridGap, 2 * gridGap + 2, 5 * gridGap, 1 * gridGap + 2); //portF (TOP-RIGHT DIAGONAL)
				}
			}
			if(this.labels[6] != null) {
				g.drawLine(4 * gridGap, 4 * gridGap, 5 * gridGap, 5 * gridGap); //portG (BOTTOM-RIGHT DIAGONAL)
				if(this.busSizes[6] > 1) {
					g.drawLine(4 * gridGap, 4 * gridGap - 1, 5 * gridGap, 5 * gridGap - 1); //portG (BOTTOM-RIGHT DIAGONAL)
					g.drawLine(4 * gridGap, 4 * gridGap + 1, 5 * gridGap, 5 * gridGap + 1); //portG (BOTTOM-RIGHT DIAGONAL)
					g.drawLine(4 * gridGap, 4 * gridGap - 2, 5 * gridGap, 5 * gridGap - 2); //portG (BOTTOM-RIGHT DIAGONAL)
					g.drawLine(4 * gridGap, 4 * gridGap + 2, 5 * gridGap, 5 * gridGap + 2); //portG (BOTTOM-RIGHT DIAGONAL)
				}
			}
			if(this.labels[7] != null) {
				g.drawLine(gridGap, 5 * gridGap, 2 * gridGap, 4 * gridGap); //portH (BOTTOM-RIGHT DIAGONAL)
				if(this.busSizes[7] > 1) {
					g.drawLine(gridGap, 5 * gridGap - 1, 2 * gridGap, 4 * gridGap - 1); //portH (BOTTOM-LEFT DIAGONAL)
					g.drawLine(gridGap, 5 * gridGap + 1, 2 * gridGap, 4 * gridGap + 1); //portH (BOTTOM-LEFT DIAGONAL)
					g.drawLine(gridGap, 5 * gridGap - 2, 2 * gridGap, 4 * gridGap - 2); //portH (BOTTOM-LEFT DIAGONAL)
				}
			}
			/*
			if (this.numPin > 6) {
				g.drawLine(gridGap, 5 * gridGap, 2 * gridGap, 4 * gridGap); //portG (BOTTOM-LEFT DIAGONAL)
				g.drawLine(gridGap, 5 * gridGap - 1, 2 * gridGap, 4 * gridGap - 1); //portG (BOTTOM-LEFT DIAGONAL)
				g.drawLine(gridGap, 5 * gridGap + 1, 2 * gridGap, 4 * gridGap + 1); //portG (BOTTOM-LEFT DIAGONAL)
				g.drawLine(gridGap, 5 * gridGap - 2, 2 * gridGap, 4 * gridGap - 2); //portG (BOTTOM-LEFT DIAGONAL)
			}*/

		}
	}
	/* ==================================================================
	 Maintanance Part
	 ================================================================= */
	private Junction portA = null; //LEFT
	private Junction portB = null; //TOP
	private Junction portC = null; //RIGHT
	private Junction portD = null; //BOTTOM
	private Junction portE = null; //TOP-LEFT
	private Junction portF = null; //BOTTOM-RIGHT
	private Junction portG = null; //DIAGONAL LEFTBOTTOM OUT (Any other bits combined into single bus - except pin 99 (special))
	private Junction portH = null; //DIAGONAL TOPRIGHT OUT single special bit
	//private Junction portO = null; //LCD OUT

	public boolean canDrop() {
		boolean result = true; //this used to be default to false but it didn't matter because of shortcuts, if things break this might be why
		if (this.labels[0] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 3, this.busSizes[0]); //PORTA
		}
		if (this.labels[1] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 3, this.gridLocation.y, this.busSizes[1]); //PORTB
		}
		if (this.labels[2] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 6, this.gridLocation.y + 3, this.busSizes[2]); //PORTC
		}
		if (this.labels[3] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 3, this.gridLocation.y + 6, this.busSizes[3]); //PORTD
		}
		if (this.labels[4] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 1, this.gridLocation.y + 1, this.busSizes[4]); //PORTE
		}
		if (this.labels[5] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 5, this.gridLocation.y + 1, this.busSizes[5]); //PORTF
		}
		if (this.labels[6] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 5, this.gridLocation.y + 5, this.busSizes[6]); //PORTG
		}
		if (this.labels[7] != null) {
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 1, this.gridLocation.y + 5, this.busSizes[7]); //PORTH
		}/*
		if (this.numPin > 4) {
			int totalbus = 0;
			for (int i = 4; i < this.busSizes.length - 10; i++) {
				totalbus = totalbus + busSizes[i];
			}
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 1, this.gridLocation.y + 5, totalbus); //portG
		}
		if(this.pin99){
			result = result && Wrapper.canDropJuncion(this.gridLocation.x + 5, this.gridLocation.y + 1, 1); //portH
		}*/
		return result;
	}

	public void droped() {
		if (this.labels[0] != null) {
			this.portA = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 3, this.busSizes[0]);
		}
		if (this.labels[1] != null) {
			this.portB = Wrapper.setPinAt(this.gridLocation.x + 3, this.gridLocation.y, this.busSizes[1]);
		}
		if (this.labels[2] != null) {
			this.portC = Wrapper.setPinAt(this.gridLocation.x + 6, this.gridLocation.y + 3, this.busSizes[2]);
		}
		if (this.labels[3] != null) {
			this.portD = Wrapper.setPinAt(this.gridLocation.x + 3, this.gridLocation.y + 6, this.busSizes[3]);
		}
		if (this.labels[4] != null) {
			this.portE = Wrapper.setPinAt(this.gridLocation.x + 1, this.gridLocation.y + 1, this.busSizes[4]);
		}
		if (this.labels[5] != null) {
			this.portF = Wrapper.setPinAt(this.gridLocation.x + 5, this.gridLocation.y + 1, this.busSizes[5]);
		}
		if (this.labels[6] != null) {
			this.portG = Wrapper.setPinAt(this.gridLocation.x + 5, this.gridLocation.y + 5, this.busSizes[6]);
		}
		if (this.labels[7] != null) {
			this.portH = Wrapper.setPinAt(this.gridLocation.x + 1, this.gridLocation.y + 5, this.busSizes[7]);
		}/*
		if (this.numPin > 4) {
			int totalbus = 0;
			for (int i = 4; i < this.busSizes.length - 5; i++) {
				totalbus = totalbus + busSizes[i];
			}
			this.portG = Wrapper.setPinAt(this.gridLocation.x + 1, this.gridLocation.y + 5, totalbus);
		}
		if(this.pin99){
			this.portH = Wrapper.setPinAt(this.gridLocation.x + 5, this.gridLocation.y + 1, 1); //portH
		}*/

		this.changeColor(Color.black);
	}

	public void selected() {
		if (this.labels[0] != null) {
			this.portA.removePin();
		}
		if (this.labels[1] != null) {
			this.portB.removePin();
		}
		if (this.labels[2] != null) {
			this.portC.removePin();
		}
		if (this.labels[3] != null) {
			this.portD.removePin();
		}
		if (this.labels[4] != null) {
			this.portE.removePin();
		}
		if (this.labels[5] != null) {
			this.portF.removePin();
		}
		if (this.labels[6] != null) {
			this.portG.removePin();
		}
		if (this.labels[7] != null) {
			this.portH.removePin();
		}/*
		if (this.numPin > 4) {
			this.portG.removePin();
		}
		if (this.pin99) {
			this.portH.removePin();
		}*/

		this.changeColor(Color.green);
	}

	public void checkAfterSelected() {
		if (this.labels[0] != null) {
			Wrapper.checkPin(this.portA);
		}
		if (this.labels[1] != null) {
			Wrapper.checkPin(this.portB);
		}
		if (this.labels[2] != null) {
			Wrapper.checkPin(this.portC);
		}
		if (this.labels[3] != null) {
			Wrapper.checkPin(this.portD);
		}
		if (this.labels[4] != null) {
			Wrapper.checkPin(this.portE);
		}
		if (this.labels[5] != null) {
			Wrapper.checkPin(this.portF);
		}
		if (this.labels[6] != null) {
			Wrapper.checkPin(this.portG);
		}
		if (this.labels[7] != null) {
			Wrapper.checkPin(this.portH);
		}/*
		if (this.numPin > 4) {
			Wrapper.checkPin(this.portG);
		}
		if (this.pin99) {
			Wrapper.checkPin(this.portH);
		}*/
	}
	/* ==================================================================
	 Simulation part
	 ================================================================= */

	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer) {
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}

	public void createEnginePeer(EnginePeerList epl) {
	}

	private Object[] loadModule(String fname) { //load components from referenced module
		String inFile = fname;
		Reader inStream;

		MainWindow.CENTRAL_PANEL.createGrid("");
		CentralPanel.ACTIVE_GRID = new Grid();

		try {
			MainWindow.CENTRAL_PANEL.createGrid("MODULE");
			MainWindow.CENTRAL_PANEL.setVisible(false);
			inStream = new BufferedReader(new FileReader(inFile));
			SaveLoadShortcut.GUI_FILE_LINK.loadMod(inStream, CentralPanel.ACTIVE_GRID);
			//loaded[next] = new loadedModule(fname,g,SaveLoadShortcut.GUI_FILE_LINK.getComp(inStream, CentralPanel.ACTIVE_GRID));
			inStream.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SimException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		//Get EnginePeers from file
		//insert into input EnginePeerList
		//connect relevant nodes

		Grid g = CentralPanel.ACTIVE_GRID;

		//*******LOAD COMPONENTS FROM FILE******
		int loop;

		int wires = g.getNumberOfWires();
		int junctions = g.getNumberOfJunctions();
		int splitters = g.getNumberOfSplitters();
		int components = g.getNumberOfComponents();
		int total;

		NodeList tempnl = new NodeList();
		EnginePeerList tempepl = new EnginePeerList();

		for (loop = 0; loop < junctions; loop++) {
			((NodeModule) g.getComponent(loop)).createNode(tempnl);
		}

		total = junctions + wires + splitters;
		for (loop = junctions + wires; loop < total; loop++) {
			((SplitterModule) g.getComponent(loop)).mergeNodes(tempnl);
		}

		total = total + components;
		//MANAGE MODULE ID
		for (loop = junctions + wires + splitters; loop < total; loop++) {
			if (g.getComponent(loop) instanceof sim.lib.others.ModuleID) {
				if (((sim.lib.others.ModuleID)g.getComponent(loop)).getModID() == 0) { //0 - use standard incremental id
					this.ModID = nextID;
					nextID++;
					((sim.lib.others.ModuleID)g.getComponent(loop)).setModID(this.ModID);
				}else if (((sim.lib.others.ModuleID)g.getComponent(loop)).getModID() < 0){ //custom incremental mod id
					if (CustomIDs.get(this.modpath) != null){
						this.ModID = CustomIDs.get(this.modpath) + 1;
						
					}else{
						this.ModID = -((sim.lib.others.ModuleID)g.getComponent(loop)).getModID();
					}
					((sim.lib.others.ModuleID)g.getComponent(loop)).setModID(this.ModID);
					CustomIDs.put(this.modpath, this.ModID);
				}else{ //custom Manual ID
					this.ModID = ((sim.lib.others.ModuleID)g.getComponent(loop)).getModID(); 
				}
				
				
			}
			((EngineModule) g.getComponent(loop)).createEnginePeer(tempepl);
		}

		Object[] out = new Object[2];

		out[0] = tempepl;
		out[1] = tempnl;

		return out;
	}

	public void createEnginePeer(EnginePeerList epl, NodeList nl) {
		//ENGINE PEER FOR MODULE
		this.busSize = 8;

		//this.ModID = nextID;
		//nextID++;

		Object[] tmp = loadModule(this.modpath); //load EPL and NL from file
		EnginePeerList tempepl = (EnginePeerList) tmp[0];
		NodeList tempnl = (NodeList) tmp[1];
		int[] pinpos = new int[this.busSizes.length]; //array to hold current bit position of each pin

		for (Object o : tempepl) {
			EnginePeer ept = (EnginePeer) o;
			if (!ept.getParent().getClass().toString().equals("class sim.lib.outputs.Pin")) {
				for (int k = 0; k < ept.getInputPins().getSize(); k++) {
					for (int i = 0; i < ept.getInputPins().getItemAt(k).getConnection().getSize(); i++) {
						EnginePeer eptt = ept.getInputPins().getItemAt(k).getConnection().getItemAt(i);
						if (eptt.getParent().getClass().toString().equals("class sim.lib.outputs.Pin")) { //component connected to a pin
							int n = ((sim.lib.outputs.Pin) eptt.getParent()).getNumber();
							//System.out.println(((sim.lib.outputs.Pin) eptt.getParent()).getLabel() + " : " + ((sim.lib.outputs.Pin) eptt.getParent()).getNumber());
							if(pinpos[n-1] >= eptt.getInputPins().size()){
								pinpos[n-1] = 0;
							}
							if (!ept.getInputPins().getItemAt(k).equals(eptt.getInputPins().getItemAt(pinpos[n - 1]))) { //find correct bit position of pin
								pinpos[n - 1] = 0;
								while (!ept.getInputPins().getItemAt(k).equals(eptt.getInputPins().getItemAt(pinpos[n - 1]))) {
									pinpos[n - 1]++;
								}
							}

							switch (n) //connect pins from loaded file to ports on module component
							{
								case 1:
									ept.setInputPin(k, this.portA.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 2:
									ept.setInputPin(k, this.portB.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 3:
									ept.setInputPin(k, this.portC.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 4:
									ept.setInputPin(k, this.portD.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 5:
									ept.setInputPin(k, this.portE.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 6:
									ept.setInputPin(k, this.portF.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 7:
									ept.setInputPin(k, this.portG.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								case 8:
									ept.setInputPin(k, this.portH.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;/*
								case 99:
									ept.setInputPin(k, this.portH.getNodes().getItemAt(pinpos[n - 1]));
									pinpos[n - 1]++;
									break;
								default:

									int base = 0;
									//n = n - 1;
									for (int j = 4; j <= (n - 2); j++) { //get correct starting bit position for portG
										base = base + this.busSizes[j];
									}
									int a = base + pinpos[n - 1];
									//System.out.println(k + "," + a);
									ept.setInputPin(k, this.portG.getNodes().getItemAt(base + pinpos[n - 1]));
									pinpos[n - 1]++;*/
							}

							break;
						}
					}
				}
				Arrays.fill(pinpos, 0); //reset array to hold current bit position of each pin
				if (ept.getOutputPins() != null) {
					for (int k = 0; k < ept.getOutputPins().getSize(); k++) {
						for (int i = 0; i < ept.getOutputPins().getItemAt(k).getConnection().getSize(); i++) {
							EnginePeer eptt = ept.getOutputPins().getItemAt(k).getConnection().getItemAt(i);
							if (eptt.getParent().getClass().toString().equals("class sim.lib.outputs.Pin")) {
								int n = ((sim.lib.outputs.Pin) eptt.getParent()).getNumber();
								if(pinpos[n-1] >= eptt.getInputPins().getSize()){
									pinpos[n-1] = 0;
								}
								if (!ept.getOutputPins().getItemAt(k).equals(eptt.getInputPins().getItemAt(pinpos[n - 1]))) { //find correct bit position of pin
									pinpos[n - 1] = 0;
									while (!ept.getOutputPins().getItemAt(k).equals(eptt.getInputPins().getItemAt(pinpos[n - 1]))) {
										pinpos[n - 1]++;
									}
								}
								switch (n) {
									case 1:
										ept.setOutputPin(k, this.portA.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 2:
										ept.setOutputPin(k, this.portB.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 3:
										ept.setOutputPin(k, this.portC.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 4:
										ept.setOutputPin(k, this.portD.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 5:
										ept.setOutputPin(k, this.portE.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 6:
										ept.setOutputPin(k, this.portF.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 7:
										ept.setOutputPin(k, this.portG.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;
									case 8:
										ept.setOutputPin(k, this.portH.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;/*
									case 99:
										ept.setOutputPin(k, this.portH.getNodes().getItemAt(pinpos[n - 1]));
										pinpos[n - 1]++;
										break;	
									default:
										int base = 0;
										for (int j = 4; j <= (n - 2); j++) { //get correct starting bit position for portG
											base = base + this.busSizes[j];
										}
										ept.setOutputPin(k, this.portG.getNodes().getItemAt(base + pinpos[n - 1]));
										pinpos[n - 1]++;*/
								}

								break;
							}
						}
					}
				}

				epl.insertItem(ept);

				if (ept.getParent() instanceof sim.lib.memory.Register) {
					if (((sim.lib.memory.Register) ept.getParent()).getRegName().equals(this.regName)) {
						//this.regout = ((sim.lib.memory.Register) ept.getParent());
						this.regOut = ((sim.lib.memory.Register) ept.getParent());
						//((sim.lib.memory.Register)ept.getParent()).setValue("07");
						//this.regList[z] = ((sim.lib.memory.Register) ept.getParent());
						//z++;
					}
					
					if (((sim.lib.memory.Register) ept.getParent()).getRegName().equals(this.initRegName)) {
						System.out.println("regval: " + this.initRegValue);
						((sim.lib.memory.Register)ept.getParent()).setValue(this.initRegValue); //set initial value
						int busSize = ((sim.lib.memory.Register)ept.getParent()).getBusSize();
						for(int i = 0; i < busSize;i++){
							ept.setOutputPinValue(busSize-i-1,(((sim.lib.memory.Register)ept.getParent()).getStore()[i]==1),0);
						}
					}
				}
				if (ept.getParent() instanceof sim.lib.memory.ShiftRegister) {
					if (((sim.lib.memory.ShiftRegister) ept.getParent()).getRegName().equals(this.regName)) {
						//this.regout = ((sim.lib.memory.Register) ept.getParent());
						this.regOut = ((sim.lib.memory.ShiftRegister) ept.getParent());
						//this.regList[z] = ((sim.lib.memory.Register) ept.getParent());
						//z++;
					}
					
					if (((sim.lib.memory.ShiftRegister) ept.getParent()).getRegName().equals(this.initRegName)) {
						System.out.println("regval: " + this.initRegValue);
						((sim.lib.memory.ShiftRegister)ept.getParent()).setValue(this.initRegValue); //set initial value
						int busSize = ((sim.lib.memory.ShiftRegister)ept.getParent()).getBusSize();
						for(int i = 0; i < busSize;i++){
							ept.setOutputPinValue(busSize-i-1,(((sim.lib.memory.ShiftRegister)ept.getParent()).getStore()[i]==1),0);
						}
					}
				}
			}

		}

		EnginePeer ep = new EnginePeer(0, 0, this);
		epl.insertItem(ep);

		for (Object n : tempnl) {
			for (Object j : ((Node) n).getJunctiions()) {
				//((Junction)j).setLocation(-100,-100);
				//((Junction)j).setEnabled(false);
			}
			for (Object w : ((Node) n).getWires()) {
				((Wire) w).setVisible(false);
			}
			EnginePeerList epltemp = ((Node) n).getConnection();
			for (Object c : epltemp) {
				((EnginePeer) c).getParent().getParentWrapper().setVisible(false);
			}
			nl.insertItem((Node) n);
		}


		//this.regOut = this.regList[regIndex];

	}

	public void reset() {
	}

	public Wrapper getParentWrapper() {
		return this;
	}

	/* ==================================================================
	 Storage Part
	 ================================================================= */
	public String getSpecificParameters() {
		return (this.modpath + Wrapper.SEPARATOR + this.regName + Wrapper.SEPARATOR + this.modLabel + Wrapper.SEPARATOR + this.colour + Wrapper.SEPARATOR + this.initRegName + Wrapper.SEPARATOR + this.initRegValue + Wrapper.SEPARATOR);
	}

	public void loadWrapper(String[] specificParameters) throws SimException {
		if (specificParameters.length == this.getNumberOfSpecificParameters()) {
			try {
				//this.setBusSize(Integer.valueOf(specificParameters[0]).intValue());
				this.setPath(specificParameters[0]);
				this.regName = specificParameters[1];
				this.modLabel = specificParameters[2];
				this.colour = specificParameters[3].equals("true");
				this.initRegName = specificParameters[4];
				this.initRegValue = specificParameters[5];
				if (specificParameters[5] == null) this.initRegValue = "";
				
				//TODO this.display = specificParamters[3];
			} catch (NumberFormatException e) {
				throw (new SimException("incorrect parameter type"));
			}
		} else {
			throw (new SimException("incorrect number of parameters"));
		}
	}

	public int getNumberOfSpecificParameters() {
		return 6;
	}

	/* ==================================================================
	 Rotation abd Flipping Part
	 ================================================================= */
	protected void adjustToChanges() {
	}
	/* ==================================================================
	 Popup Part
	 ================================================================= */
	private int oldBusSize = 0;

	public boolean hasProperties() {
		return true;
	}

	public Component getPropertyWindow() {
		return (new ModProperties(new String[5], this.regList, this.regName, this.modLabel, this.colour, this.initRegName, this.initRegValue));
	}

	public void respondToChanges(Component property) {
		CentralPanel.ACTIVE_GRID.eraseComponent(this, false);
		ModProperties mp = (ModProperties) property;
		this.regName = mp.getRegName();
		this.modLabel = mp.getLabel();
		this.colour = mp.getColour();
		this.initRegName = mp.getInitRegName();
		this.initRegValue = mp.getInitRegValue();
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}

	public void restoreOriginalProperties() {
		if (this.oldBusSize != 0) {
			this.setBusSize(this.oldBusSize);
			this.oldBusSize = 0;
		}
	}

	@Override
	public void writeVHDLEntity(String path) {
		//Nothing - components of module write vhdl themselves
		
	}

	@Override
	public String getVHDLName() {
		//nothing
		return null;
	}
	
	
}
