package sim.lib.others;

import java.awt.*;

import sim.*;
import sim.engine.*;
import sim.lib.EditBusSize;
import sim.lib.outputs.PinProperties;
import sim.lib.wires.Junction;

public class TextBox extends WrapperPainted implements EngineModule {
	/* ==================================================================
	 Creation Part
	 ================================================================= */

	private static Image ICON = GuiFileLink.getImage("sim/lib/others/TextBox.gif");

	public Image getIcon() {
		return this.ICON;
	}

	public Wrapper createWrapper() {
		return this.getCopy();
	}

	public Wrapper createWrapper(Point gridPosition) {
		TextBox result = (TextBox) this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}

	public String getBubbleHelp() {
		return "Text Box";
	}
	/* ==================================================================
	 GUI part
	 ================================================================= */
	private String text = "Text";
	private int fontSize;
	private String fontName;
	private String fontStyle;

	public TextBox() {
		super();
	}

	protected Wrapper getCopy() {
		TextBox result = new TextBox();
		result.text = this.text;
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}

	public void initializeGridSize() {
		this.setGridSize(2, 1);
	}

	public void setContent(String fontName, String fontStyle, int fontSize, String text) {
		this.fontName = fontName;

	}

	/* ==================================================================
	 Maintanance Part
	 ================================================================= */
	private Junction input = null;
	
	public void selected() {
		this.changeColor(Color.green);
	}

	public void checkAfterSelected() {
		Wrapper.checkPin(this.input);
	}
	
	/* ==================================================================
	 Simulation part
	 ================================================================= */

	public void evaluateOutput(double currentTime, Data[] currentInputs, EnginePeer peer) {
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}

	public void createEnginePeer(EnginePeerList epl) {
		EnginePeer ep = new EnginePeer(this);
		epl.insertItem(ep);
	}

	public void reset() {
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}

	public Wrapper getParentWrapper() {
		return this;
	}

	/* ==================================================================
	 Storage Part
	 ================================================================= */
	public String getSpecificParameters() {
		return (this.fontName + Wrapper.SEPARATOR + this.fontStyle + Wrapper.SEPARATOR + Integer.toString(this.fontSize) + Wrapper.SEPARATOR + this.text + Wrapper.SEPARATOR);
	}

	public void loadWrapper(String[] specificParameters) throws SimException {
		if (specificParameters.length == this.getNumberOfSpecificParameters()) {
			try {
			} catch (NumberFormatException e) {
				throw (new SimException("incorrect parameter type"));
			}
		} else {
			throw (new SimException("incorrect number of parameters"));
		}
	}

	public int getNumberOfSpecificParameters() {
		return 4;
	}

	/* ==================================================================
	 Rotation abd Flipping Part
	 ================================================================= */
	public void paint(Graphics g) {
		g.setColor(this.brush);
		int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
		g.setFont(new Font(Wrapper.FONT_NAME,Font.PLAIN, gridGap));
		int textWidth = g.getFontMetrics(g.getFont()).stringWidth(text);
		this.setGridSize(textWidth / 13, 1);
		g.drawString(text,0,gridGap);
	}
	
	public boolean canDrop() {
		return true;
	}

	public void droped() {
		this.changeColor(Color.black);
	}

	protected void adjustToChanges() {
	}
	/* ==================================================================
	 Popup Part
	 ================================================================= */
	private int oldBusSize = 0;

	public boolean hasProperties() {
		return true;
	}

	public Component getPropertyWindow() {
		return (new TextBoxProperties(this.text));
	}

	public void respondToChanges(Component property) {
		this.brush = Color.white;
		CentralPanel.ACTIVE_GRID.paintComponent(this); //ERASE OLD DRAWING
		this.brush = Color.green;
		this.text = (((TextBoxProperties)property).getText());
		if(this.text.equals(""))
		{
			this.text = "Text";
		}
		CentralPanel.ACTIVE_GRID.paintComponent(this);
	}

	public void restoreOriginalProperties() {
	}

	@Override
	public void writeVHDLEntity(String path) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public String getVHDLName() {
		// TODO Auto-generated method stub
		return null;
	}
}
