package sim.lib.outputs;

import java.awt.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.Writer;
import java.util.Scanner;

import sim.*;
import sim.engine.*;
import sim.lib.wires.Junction;

public class SevenSegmentDecoder extends RotatableFlippableWrapperPainted implements EngineModule {
	/* ==================================================================
	 Creation Part
	 ================================================================= */

	private static Image ICON = GuiFileLink.getImage("sim/lib/outputs/SevenSegmentDecoder.gif");

	public Image getIcon() {
		return ICON;
	}

	public Wrapper createWrapper() {
		return this.getCopy();
	}

	public Wrapper createWrapper(Point gridPosition) {
		SevenSegmentDecoder result = this.getCopy();
		result.setGridLocation(gridPosition);
		return result;
	}

	public String getBubbleHelp() {
		return "7 segment decoder";
	}

	public void loadWrapper(String[] paramArrayOfString)
			throws SimException {
		if (paramArrayOfString.length == getNumberOfSpecificParameters()) {
			if (!paramArrayOfString[0].equals("null")) {
				throw new SimException("incorrect parameter type");
			}
		} else {
			throw new SimException("incorrect number of parameters");
		}
	}

	/* ==================================================================
	 GUI part
	 ================================================================= */

	public SevenSegmentDecoder() {
		super();
	}
	
	public SevenSegmentDecoder getCopy()
	{
		SevenSegmentDecoder result = new SevenSegmentDecoder();
		
		result.setGridLocation(this.getGridLocation());
		result.setGridSize(this.getGridSize());
		result.setAngle(this.getAngle());
		result.setFlipped(this.isFlipped());
		
		return result;
	}

	public void initializeGridSize() {
		setGridSize(4, 4);
	}
	/* ==================================================================
	 Maintanance Part
	 ================================================================= */
	private Junction input = new Junction(1);
	private Junction output = new Junction(1);
	private boolean[] value = new boolean[7];

	public void selected() {
		this.input.removePin();
		this.output.removePin();
		changeColor(Color.green);
	}

	public void checkAfterSelected() {
		Wrapper.checkPin(this.input);
		Wrapper.checkPin(this.output);
	}
	
	/* ==================================================================
	 Simulation part
	 ================================================================= */

	public void evaluateOutput(double paramDouble, Data[] paramArrayOfData, EnginePeer peer) {
		this.value[0] = paramArrayOfData[3].getValue() && paramArrayOfData[1].getValue();
		this.value[0] = this.value[0] || (paramArrayOfData[2].getValue() && paramArrayOfData[0].noValue());
		this.value[0] = this.value[0] || (paramArrayOfData[1].noValue() && paramArrayOfData[0].noValue());
		this.value[0] = this.value[0] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue());
		this.value[0] = this.value[0] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].getValue() && paramArrayOfData[1].noValue());
		this.value[1] = paramArrayOfData[3].getValue() && paramArrayOfData[1].getValue();
		this.value[1] = this.value[1] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].getValue());
		this.value[1] = this.value[1] || (paramArrayOfData[3].getValue() && paramArrayOfData[0].noValue());
		this.value[1] = this.value[1] || (paramArrayOfData[1].getValue() && paramArrayOfData[0].noValue());
		this.value[1] = this.value[1] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue() && paramArrayOfData[0].noValue());
		this.value[2] = paramArrayOfData[3].noValue() && paramArrayOfData[1].getValue();
		this.value[2] = this.value[2] || (paramArrayOfData[2].getValue() && paramArrayOfData[1].getValue());
		this.value[2] = this.value[2] || (paramArrayOfData[3].getValue() && paramArrayOfData[0].noValue());
		this.value[2] = this.value[2] || (paramArrayOfData[1].getValue() && paramArrayOfData[0].noValue());
		this.value[2] = this.value[2] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue() && paramArrayOfData[0].noValue());
		this.value[2] = this.value[2] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].getValue() && paramArrayOfData[0].getValue());
		this.value[2] = this.value[2] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue() && paramArrayOfData[1].noValue());
		this.value[3] = paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue();
		this.value[3] = this.value[3] || (paramArrayOfData[1].getValue() && paramArrayOfData[0].noValue());
		this.value[3] = this.value[3] || (paramArrayOfData[3].getValue() && paramArrayOfData[0].getValue());
		this.value[3] = this.value[3] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].getValue() && paramArrayOfData[1].noValue());
		this.value[3] = this.value[3] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue() && paramArrayOfData[1].getValue());
		this.value[4] = paramArrayOfData[3].getValue() && paramArrayOfData[1].noValue();
		this.value[4] = this.value[4] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue() && paramArrayOfData[0].getValue());
		this.value[4] = this.value[4] || (paramArrayOfData[2].getValue() && paramArrayOfData[1].noValue() && paramArrayOfData[0].getValue());
		this.value[4] = this.value[4] || (paramArrayOfData[2].getValue() && paramArrayOfData[1].getValue() && paramArrayOfData[0].noValue());
		this.value[4] = this.value[4] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue() && paramArrayOfData[1].getValue());
		this.value[4] = this.value[4] || (paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue() && paramArrayOfData[0].noValue());
		this.value[5] = paramArrayOfData[3].noValue() && paramArrayOfData[2].noValue();
		this.value[5] = this.value[5] || (paramArrayOfData[2].noValue() && paramArrayOfData[1].noValue());
		this.value[5] = this.value[5] || (paramArrayOfData[3].noValue() && paramArrayOfData[1].noValue() && paramArrayOfData[0].noValue());
		this.value[5] = this.value[5] || (paramArrayOfData[3].noValue() && paramArrayOfData[1].getValue() && paramArrayOfData[0].getValue());
		this.value[5] = this.value[5] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue() && paramArrayOfData[0].noValue());
		this.value[5] = this.value[5] || (paramArrayOfData[3].getValue() && paramArrayOfData[1].noValue() && paramArrayOfData[0].getValue());
		this.value[6] = paramArrayOfData[3].noValue() && paramArrayOfData[2].getValue();
		this.value[6] = this.value[6] || (paramArrayOfData[3].getValue() && paramArrayOfData[2].noValue());
		this.value[6] = this.value[6] || (paramArrayOfData[1].noValue() && paramArrayOfData[0].getValue());
		this.value[6] = this.value[6] || (paramArrayOfData[2].noValue() && paramArrayOfData[1].noValue());
		this.value[6] = this.value[6] || (paramArrayOfData[2].noValue() && paramArrayOfData[0].getValue());
		for(int i = 0; i < 7; i ++) {
			peer.setOutputPinValue(i, this.value[i], paramDouble);
		}
	}

	public void createEnginePeer(EnginePeerList paramEnginePeerList) {
		EnginePeer localEnginePeer;
		localEnginePeer= new EnginePeer(4, 7, this);
		int i = 0;
		do {
			localEnginePeer.setInputPin(i, this.input.getNodes().getItemAt(i));
			i++;
		} while (i < 4);
		i = 0;
		do {
			localEnginePeer.setOutputPin(i, this.output.getNodes().getItemAt(i));
			i++;
		} while (i < 7);
		paramEnginePeerList.insertItem(localEnginePeer);
	}

	public void reset() {
	}

	public Wrapper getParentWrapper() {
		return this;
	}

	/* ==================================================================
	 Storage Part
	 ================================================================= */

	public String getSpecificParameters() {
		return "null" + Wrapper.SEPARATOR;
	}

	public int getNumberOfSpecificParameters() {
		return 1;
	}
	
	/* ==================================================================
	 GUI part
   	 ================================================================= */

	protected void paintNormal_0(Graphics g) {
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawRect(gridGap, gridGap, twice, twice);
			
			g.fillRect(twice - 1, 0, 3, gridGap);
			g.fillRect(twice - 1, 3 * gridGap, 3, gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("7 seg", twice - fm.stringWidth("7 seg") / 2, twice + fm.getAscent() / 2);
			g.drawString("4", twice + increment, gridGap - increment);
			g.drawString("7", twice + increment, 4 * gridGap - increment);
		}
	}

	protected void paintNormal_90(Graphics g) {
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawRect(gridGap, gridGap, twice, twice);
			
			g.fillRect(0, twice - 1, gridGap, 3);
			g.fillRect(3 * gridGap, twice - 1, gridGap, 3);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("7 seg", twice - fm.stringWidth("7 seg") / 2, twice + fm.getAscent() / 2);
			g.drawString("4", increment, twice - increment);
			g.drawString("7", 3 * gridGap + increment, twice - increment);
		}
	}

	protected void paintNormal_180(Graphics g) {
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawRect(gridGap, gridGap, twice, twice);
			
			g.fillRect(twice - 1, 0, 3, gridGap);
			g.fillRect(twice - 1, 3 * gridGap, 3, gridGap);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("7 seg", twice - fm.stringWidth("7 seg") / 2, twice + fm.getAscent() / 2);
			g.drawString("7", twice + increment, gridGap - increment);
			g.drawString("4", twice + increment, 4 * gridGap - increment);
		}
	}

	protected void paintNormal_270(Graphics g) {
		// draw if visible
		if(this.isVisible())
		{
			int gridGap = CentralPanel.ACTIVE_GRID.getCurrentGridGap();
			int increment = gridGap / 4;
			int twice = 2 * gridGap;
			
			g.setColor(this.brush);
			
			g.drawRect(gridGap, gridGap, twice, twice);
			
			g.fillRect(0, twice - 1, gridGap, 3);
			g.fillRect(3 * gridGap, twice - 1, gridGap, 3);
			
			g.setFont(new Font(Wrapper.FONT_NAME, Font.PLAIN, 3 * increment));
			
			FontMetrics fm = g.getFontMetrics(g.getFont());
			g.drawString("7 seg", twice - fm.stringWidth("7 seg") / 2, twice + fm.getAscent() / 2);
			g.drawString("7", increment, twice - increment);
			g.drawString("4", 3 * gridGap + increment, twice - increment);
		}
	}

	protected void paintFlipped_0(Graphics paramGraphics) {
		paintNormal_180(paramGraphics);
	}

	protected boolean canDropFlipped_0() {
		return canDropNormal_180();
	}

	protected void dropedFlipped_0() {
		dropedNormal_180();
	}

	protected void dropedNormal_0() {
		this.input = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y, 4);
		this.output = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y + 4, 7);
		changeColor(Color.black);
	}

	protected boolean canDropNormal_0() {
		boolean result;
		result = Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y + 4, 7);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y, 4);
		return result;
	}

	protected void paintFlipped_90(Graphics paramGraphics) {
		paintNormal_90(paramGraphics);
	}

	protected boolean canDropFlipped_90() {
		return canDropNormal_90();
	}

	protected void dropedFlipped_90() {
		dropedNormal_90();
	}

	protected boolean canDropNormal_90() {
		boolean result;
		result = Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 2, 4);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 4, this.gridLocation.y + 2, 7);
		return result;
	}

	protected void dropedNormal_90() {
		this.input = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 2, 4);
		this.output = Wrapper.setPinAt(this.gridLocation.x + 4, this.gridLocation.y + 2, 7);
		changeColor(Color.black);
	}

	protected void adjustToChanges() {
	}

	protected void paintFlipped_180(Graphics paramGraphics) {
		paintNormal_0(paramGraphics);
	}

	protected boolean canDropFlipped_180() {
		return canDropNormal_0();
	}

	protected void dropedFlipped_180() {
		dropedNormal_0();
	}

	protected boolean canDropNormal_180() {
		boolean result;
		result = Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y, 7);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 2, this.gridLocation.y + 4, 4);
		return result;
	}

	protected void dropedNormal_180() {
		this.input = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y + 4, 4);
		this.output = Wrapper.setPinAt(this.gridLocation.x + 2, this.gridLocation.y, 7);
		changeColor(Color.black);
	}

	protected boolean canDropFlipped_270() {
		return canDropNormal_270();
	}

	protected void dropedFlipped_270() {
		dropedNormal_270();
	}

	protected void paintFlipped_270(Graphics paramGraphics) {
		paintNormal_270(paramGraphics);
	}

	protected boolean canDropNormal_270() {
		boolean result;
		result = Wrapper.canDropJuncion(this.gridLocation.x, this.gridLocation.y + 2, 7);
		result = result && Wrapper.canDropJuncion(this.gridLocation.x + 4, this.gridLocation.y + 2, 4);
		return result;
	}

	protected void dropedNormal_270() {
		this.input = Wrapper.setPinAt(this.gridLocation.x + 4, this.gridLocation.y + 2, 4);
		this.output = Wrapper.setPinAt(this.gridLocation.x, this.gridLocation.y + 2, 7);
		changeColor(Color.black);
	}

	@Override
	public void writeVHDLEntity(String path) {
		String fileName = "gates.vhd";
		File file = new File(path+"\\"+fileName);
		boolean written = false;
		try {
			Scanner scanner = new Scanner(file);
			while(scanner.hasNextLine()){
				String line = scanner.nextLine();
				if(line.contains(this.getVHDLName())){
					written = true;
				}
			}
			scanner.close();
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
		}
		
		if(!written){
			Writer writer = null;
			try {
				writer = new FileWriter(path+"\\"+fileName,true);
				writer.write("\nlibrary IEEE;\n");
				writer.write("use IEEE.STD_LOGIC_1164.ALL;\n");
				
				writer.write("\nentity " + this.getVHDLName() + " is\n");
				writer.write("port(\n");
				for(int x = 0; x < 4; x++)
					writer.write("x" + x + ": in std_logic;\n");
				for(int f = 0; f < 6; f++)
					writer.write("f" + f + ": out std_logic;\n");
				writer.write("f6: out std_logic);\n");
				writer.write("end "+ this.getVHDLName() + ";\n\n");

				writer.write("architecture behav of "+ this.getVHDLName() + " is\n");
				writer.write("begin\n");
				writer.write("f0 <= '1' when (x1 = '0' and x0 = '0') or (x3 = '1' and x1 = '1') or (x3 = '1' and x2 = '0') or (x2 = '1' and x0 = '0') or (x3 = '0' and x2 = '1' and x1 = '0') else '0';\n");
				writer.write("f1 <= '1' when (x3 = '1' and x2 = '1') or (x3 = '1' and x1 = '1') or (x3 = '1' and x0 = '0') or (x1 = '1' and x0 = '0') or (x3 = '0' and x2 = '0' and x0 = '0') else '0';\n");
				writer.write("f2 <= '1' when (x1 = '1' and x0 = '0') or (x3 = '1' and x0 = '0') or (x2 = '1' and x1 = '1') or (x3 = '0' and x1 = '1') or (x3 = '1' and x2 = '0' and x1 = '0') or (x3 = '0' and x2 = '1' and x0 = '1') or (x3 = '0' and x2 = '0' and x0 = '0') else '0';\n");
				writer.write("f3 <= '1' when (x3 = '1' and x2 = '0') or (x1 = '1' and x0 = '0') or (x3 = '1' and x0 = '1') or (x3 = '0' and x2 = '1' and x1 = '0') or (x3 = '0' and x2 = '0' and x1 = '1') else '0';\n");
				writer.write("f4 <= '1' when (x3 = '1' and x1 = '0') or (x3 = '1' and x2 = '0' and x0 = '1') or (x2 = '1' and x1 = '0' and x0 = '1') or (x2 = '1' and x1 = '1' and x0 = '0') or (x3 = '0' and x2 = '0' and x1 = '1') or (x3 = '0' and x2 = '0' and x0 = '0') else '0';\n");
				writer.write("f5 <= '1' when (x3 = '0' and x2 = '0') or (x2 = '0' and x1 = '0') or (x3 = '0' and x1 = '0' and x0 = '0') or (x3 = '0' and x1 = '1' and x0 = '1') or (x3 = '1' and x2 = '0' and x0 = '0') or (x3 = '1' and x1 = '0' and x0 = '1') else '0';\n");
				writer.write("f6 <= '1' when (x3 = '0' and x2 = '1') or (x3 = '1' and x2 = '0') or (x1 = '0' and x0 = '1') or (x2 = '0' and x1 = '0') or (x2 = '0' and x0 = '1') else '0';\n");
				writer.write("end behav;\n");
				writer.close();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		
	}

	@Override
	public String getVHDLName() {
		// TODO Auto-generated method stub
		return "SevenSegmentEncoder";
	}
}
